/**************************************************************************** 
* This source code was originally from AcQuisition Technologies.
*	C&H made minor modifications to integrate it into its driver architecture.
*
*	C&H Part #: 11029467
*****************************************************************************/


/*
 *  file:     m392lib.c
 *  revision: 1.1
 *  date:     03/01/00
 *  author:   JG
 * ----------------------------------------------------------------------
 *
 * M392 12-/16-bit ADC M-module with 16x Single Ended Inputs
 * APIS Based ANSI-C Library
 *
 * APIS is AcQuisition Technology's Platform Interface Software.
 * The library software in combination with the demo application
 * (refer to source file: m392demo.c) serve as an example for
 * writing application software for the M392.
 * ----------------------------------------------------------------------
 * Copyright 2000-2001 by AcQuisition Technology bv (c)
 * All Rights Reserved
 * Reproduced Under License
 *
 * This source code is the proprietary confidential property of
 * AcQuisition Technology bv, and is provided to the licensee
 * for documentation and educational purposes only.  Reproduction,
 * publication, or any form of distribution to any party other than
 * the licensee is strictly prohibited.
 * ----------------------------------------------------------------------
 * Edition History
 *
 * #    date        Comments                                        by
 * --   --------    ----------------------------------------------  ---
 * 1.0  03/01/00    First release                                   JG
 * 1.1  03/05/01    m392_ramboot() added                            SP
 *
 */
#include "apis.h"
#include "m392defs.h"

/* MACRO BOOT_FROM_RAM
 * If the macro BOOT_FROM_RAM is defined as TRUE the
 * function m392_ramboot() will be available.
 * With the function m392_ramboot a firmware image will be downloaded
 * to the module and will be loaded from RAM rather than from ROM
 * For this purpose a source file containing the firmware image
 * must be linked to the program; the Firmware image can be obtained
 * from AcQuisition Technology on request.
 *
 */

/*
 * Forward declarations
 */
/* booting m392 */
int m392_boot (APIS_HANDLE handle);
/* load boot image from RAM */
int m392_ramboot (APIS_HANDLE handle);
/* give command to m392 */
int m392_cmd (APIS_HANDLE handle, UINT16 command, UINT16 *pArgList,
                int ni, int no);
/* unmask interrupt and install interrupt handler(m392_irqh) on m392 */
int m392_install_irq( APIS_HANDLE handle);
/* mask interrupt and remove interrupt handler */
int m392_remove_irq( APIS_HANDLE handle);
/* interrupt handler */
int m392_irqh (APIS_HANDLE handle);
/* setup channel configuration */
int m392_setupchan(APIS_HANDLE handle, UINT16 *ctrlwrdlist);
/* setup all channels with same input range */
int m392_setupallchan(APIS_HANDLE handle, UINT16 cntlwrd);
/* setup acquisition time */
int m392_setupacqtime( APIS_HANDLE handle, UINT16 throughput);
/* configure moving average filter */
int m392_configmafilter( APIS_HANDLE handle, UINT16 filter_beh);
/* read data from one channel */
int m392_readchannel( APIS_HANDLE handle, UINT16 channel, UINT16 *data);
/* Check RDY bit */
int m392_check_rdybit( APIS_HANDLE handle);
/* wait till RDY bit is set */
void m392_wait4rdybit( APIS_HANDLE handle);
/* clear RDY bit */
void m392_clear_rdybit( APIS_HANDLE handle);

/*
 * Externals
 */
#if BOOT_FROM_RAM
extern const unsigned char m392firm[];  /* firmware image */
extern const unsigned m392firm_sz;      /* firmware size in bytes */
#endif

/*
 * Function:    m392_boot
 *
 * Description: Load bootfile from on-board ROM and boot
 *
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *
 * Returns:     0 or -1 in case of hardware failure
 *
 */
int m392_boot (APIS_HANDLE handle)
{
    UINT16 data;

    /* Put module in reset */
    apis_write(handle, sizeof(UINT16), CTRLREG, (UINT16)(RESET));


    /* Release module reset and boot from ROM */
    apis_write(handle, sizeof(UINT16), CTRLREG, (UINT16)0);

    /* Wait until ready bit is set */
    do
        apis_read(handle, sizeof(UINT16), CTRLREG, &data);
    while ( !( data & RDY));

    /* Write 0x100 in page register-> command page in dual ported
       memory */
    apis_write(handle, sizeof(UINT16), PAGEREG, (UINT16)(CMDPAGE));


    /* Read global status */
    apis_read(handle, sizeof(UINT16), GLOBSTAT, &data);

    if ( data == 0 )
        return NOERR;  /* M392 ready for usage */
    else
        return -1; /* Hardware failure */
}


/*
 * Function:    m392_ramboot
 *
 * Description: Download bootfile to dual port memory, the DSP will
 *              load the bootfile from RAM.
 *              To enable this function the macro BOOT_FROM_RAM
 *              must be defined as TRUE and a boot-image must be
 *              linked to the application
 *
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *
 * Returns:     0 or -1 in case of hardware failure
 *
 */
#if BOOT_FROM_RAM
int m392_ramboot (APIS_HANDLE handle)
{
    UINT16 data;
    UINT32 i, j, size;
    UINT8 *ptr;

    /* Put module in reset */
    apis_write(handle, sizeof(UINT16), CTRLREG, (UINT16)(RESET));

    /* Set boot bit -> booting from RAM */
    apis_write(handle, sizeof(UINT16), CTRLREG, (UINT16)(BOOT+RESET));

    ptr = (UINT8 *)&m392firm[0];    /* get pointer to boot image */
    size = (m392firm_sz+128)/128;   /* get size in pages */

    /* Download firmware image to dual ported memory */
    for (i = 0; i < size; i++)
    {
        apis_write(handle, sizeof(UINT16), PAGEREG, (UINT16)(i));
        for (j = 0; j < 64; j++)
        {
            data = (UINT16)(*ptr++ << 8);
            data |= (UINT16)*ptr++;
            apis_write(handle, sizeof(UINT16), (2*j), (UINT16)(data));
        }
    }

    /* Release module reset and boot from RAM */
    apis_write(handle, sizeof(UINT16), CTRLREG, (UINT16)BOOT);

    /* Wait until ready bit is set */
    do
        apis_read(handle, sizeof(UINT16), CTRLREG, &data);
    while ( !( data & RDY));

    /* Write 0x100 in page register-> command page in dual ported
       memory */
    apis_write(handle, sizeof(UINT16), PAGEREG, (UINT16)(CMDPAGE));


    /* Read global status */
    apis_read(handle, sizeof(UINT16), GLOBSTAT, &data);

    if ( data == 0)
        return NOERR;  /* M392 ready for usage */
    else
        return -1; /* Hardware failure */
}
#endif


/*
 * Function:    m392_cmd
 *
 * Description: Copy command parameters to the parameter field in
 *              shared ram of the module. Execute command and wait
 *              for the command to be executed.
 *              Copy the result parameters and return with the
 *              result code obtained from the firmware.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              UINT16 command
 *                  firmware command
 *              void *pArgList
 *                  pointer to the command parameter list
 *              int ni
 *                  number of input command parameters
 *              int no
 *                  number of output command parameters
 *
 *
 * Returns:     the command result code
 *
 */
int m392_cmd (APIS_HANDLE handle, UINT16 command, UINT16 *pArgList,
                int ni, int no)
{
    UINT16 result;
    UINT16 data;
    int i;

    /* Write 0x100 to page register -> command page in dual ported
       memory */
    apis_write(handle, sizeof(UINT16), PAGEREG, (UINT16)(CMDPAGE));

    /* Write input parameter fields */
    for ( i=0; i < ni; i++)
        apis_write(handle, sizeof(UINT16), (C_PRM0+(2*i)),
                   (UINT16)(pArgList[i]));

    /* Write command to command field */
    apis_write(handle, sizeof(UINT16), C_CMD, (UINT16)(command));

    /* Wait till command is executed */
    do
        apis_read( handle, sizeof(UINT16), C_CMD, &data);
    while (data);

    /* Read result from result field */
    apis_read(handle, sizeof(UINT16), C_RES, &result);

    if ( result == 0)
        /* Read output parameters from parameter fields */
        for ( i=0; i < no; i++)
            apis_read( handle, sizeof(UINT16), (C_PRM0+(2*i)), &pArgList[i]);

    return (int)(result);
}

/*
 * Function:    m392_install_irqrdy
 *
 * Description: Install interrupt handler and unmask interrupt
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *
 * Returns:     Returns 0 or -1 in case of error installing the interrupt
 *              handler
 *
 */
int m392_install_irq( APIS_HANDLE handle)
{
    /* install interrupt handler */
    if (apis_irqinstall(handle, (void *)m392_irqh, 0,0,0,0))
        return -1;

    /* unmask interrupt */
    apis_write( handle, sizeof(UINT16), IMASK, 0x8000);

    return (NOERR);
}

/*
 * Function:    m392_remove_irq
 *
 * Description: Removes interrupt handler and masks interrupt
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *
 * Returns:     Returns 0
 *
 */
int m392_remove_irq( APIS_HANDLE handle)
{
    /* mask interrupt */
    apis_write(handle, sizeof(UINT16), IMASK, 0x8000);

    /* remove interrupt handler */
    apis_irqremove( handle);

    return (NOERR);
}

/*
 * Function:    m392_irqh
 *
 * Description: The interrupt handler will clear the interrupt of
 *              the m392. Pending interrupts are cleared with the IACK
 *              command.
 *
 * Parameters:  none
 *
 * Returns:     0 or -1 if interrupt is not ours
 */
int m392_irqh (APIS_HANDLE handle)
{
    UINT16 data;

    /* Read control register */
    apis_read(handle, sizeof(UINT16), CTRLREG, &data);

    if (data & HIRQ) {         /* is interrupt ours ? */
        /* clear interrupt */
        apis_write(handle, sizeof(UINT16), CTRLREG, (UINT16)(HIRQ));

        /* Enter code which has to be executed when this
           interrupt occurs here */

        return 0;
    }
    return -1;
}

/*
 * Function:    m392_setupchan
 *
 * Description: Puts input ranges of channel 0-15 in parameter fields
 *              0-15 and puts setup command in command field.
 *              Waits till this command is executed and returns the
 *              result of it.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              UNINT16 *inp_range_list
 *                  pointer to an array with input ranges for channel
 *                  0-15
 *
 * Returns:     the command result code
 */
int m392_setupchan(APIS_HANDLE handle, UINT16 *inp_range_list)
{
    UINT16 result, data;
    int i;

    /* Write control words to parameter fields */
    for ( i=0; i<16; i++)
        apis_write( handle, sizeof(UINT16), C_PRM0+(2*i),
                   (UINT16)(inp_range_list[i]));

    /* Put 'setup channel configuration' command in command
       field */
    apis_write( handle, sizeof(UINT16), C_CMD, (UINT16)(SETUP));

    /* Wait till command is executed */
    do
        apis_read( handle, sizeof(UINT16), C_CMD, &data);
    while (data);

    /* Read result from result field */
    apis_read(handle, sizeof(UINT16), C_RES, &result);
    return (int)(result);
}

/*
 * Function:    m392_setupallchan
 *
 * Description: Configures channel 0-15 with the same input range.
 *              Fills parameter field 0-15 with input parameter
 *              inp_range and puts setup command in command field.
 *              Waits till this command is executed and returns the
 *              result of it.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              UNINT16 inp_range
 *                  input range for channel 0-15
 *
 * Returns:     the command result code
 */
int m392_setupallchan(APIS_HANDLE handle, UINT16 inp_range)
{
    UINT16 result, data;
    int i;

    /* Write input range in parameter field 0-15 */
    for ( i=0; i<16; i++)
        apis_write( handle, sizeof(UINT16), C_PRM0+(2*i), (UINT16)(inp_range));

    /* Put 'setup channel configuration' command in command field */
    apis_write( handle, sizeof(UINT16), C_CMD, (UINT16)(SETUP));

    /* Wait till command is executed */
    do
        apis_read( handle, sizeof(UINT16), C_CMD, &data);
    while (data);

    /* Read result from result field */
    apis_read(handle, sizeof(UINT16), C_RES, &result);
    return (int)(result);
}

/*
 * Function:    m392_setupacqtime
 *
 * Description: Setup the acquisition time.
 *              Puts throughput in parameter field 0 and puts
 *              command for setting up acquisition time in command
 *              field. Waits till this command is executed and
 *              returns the result of it.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              UNINT16 throughput
 *                  throughput of acquisition data
 *
 * Returns:     the command result code
 */
int m392_setupacqtime( APIS_HANDLE handle, UINT16 throughput)
{
    UINT16 result, data;

    /* Write throughput in parameter field 0 */
    apis_write( handle, sizeof(UINT16), C_PRM0, (UINT16)(throughput));

    /* Put 'setup acquisition time' command in command field */
    apis_write( handle, sizeof(UINT16), C_CMD, (UINT16)(SETACQTIME));

    /* Wait till command is executed */
    do
        apis_read( handle, sizeof(UINT16), C_CMD, &data);
    while (data);

    /* Read result from result field */
    apis_read(handle, sizeof(UINT16), C_RES, &result);
    return (int)(result);
}

/*
 * Function:    m392_configmafilter
 *
 * Description: Configure moving average filter.
 *              Puts filter behavior in parameter field 0 and
 *              puts command to configure MA filter in command
 *              field. Waits till command is executed and returns
 *              the result of it.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              UNINT16 filter_beh
 *                  filter behavior
 *
 * Returns:     the command result code
 */
int m392_configmafilter( APIS_HANDLE handle, UINT16 filter_beh)
{
    UINT16 result, data;

    /* Write MA filter behavior in parameter field 0 */
    apis_write( handle, sizeof(UINT16), C_PRM0, (UINT16)(filter_beh));

    /* Put 'configure MA filter' command in command field */
    apis_write( handle, sizeof(UINT16), C_CMD, (UINT16)(MAFILTER));

    /* Wait till command is executed */
    do
        apis_read( handle, sizeof(UINT16), C_CMD, &data);
    while (data);

    /* Read result from result field */
    apis_read(handle, sizeof(UINT16), C_RES, &result);
    return (int)(result);
}

/*
 * Function:    m392_readchannel
 *
 * Description: Reads data from one channel
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              UINT8 channel
 *                  Channel number
 *              UINT16 *data
 *                  Pointer to data from channel
 *
 * Returns:     0 or -1 on error
 *
 */
int m392_readchannel( APIS_HANDLE handle, UINT16 channel, UINT16 *data)
{
    /* Read data from one channel */
    if ( apis_read( handle, sizeof(UINT16), (UINT16)(MDR0+(2*channel)), data)
        != 0)
        return -1;
    return NOERR;
}

/*
 * Function:    m392_check_rdybit
 *
 * Description: Check if RDY bit is set
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *
 * Returns:     TRUE when RDY bit is set and FALSE when it is not set
 *
 */
int m392_check_rdybit( APIS_HANDLE handle)
{
    UINT16 data;

    apis_read( handle, sizeof(UINT16), (UINT16)CTRLREG, &data);
    if ( data & RDY)
       return TRUE;
    else return FALSE;
}


/*
 * Function:    m392_wait4rdybit
 *
 * Description: Wait till RDY bit is set
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *
 * Returns:     Nothing
 *
 */
void m392_wait4rdybit( APIS_HANDLE handle)
{
    UINT16 data;

    do {
        apis_read( handle, sizeof(UINT16), (UINT16)CTRLREG, &data);
    } while ( !(data & RDY));
}

/*
 * Function:    m392_clear_rdybit
 *
 * Description: Clears RDY bit
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *
 * Returns:     Nothing
 *
 */
void m392_clear_rdybit( APIS_HANDLE handle)
{
    apis_write( handle, sizeof(UINT16), (UINT16)CTRLREG, RDY);
}

